//! Rust Challenge: Satellite Propulsion Sim

mod project;



use std::io;

use tokio::sync::watch;



type IoResult<T> = Result<T, io::Error>;



/// Print the version of the project.
fn print_version()
{
   println!("{} v{}", project::get_name(), project::get_version());
}

/// Asynchronously listen for CTRL-C to be pressed.
///
/// Sends `false` through the watch channel when the signal is received.
async fn listen_for_termination(sender: watch::Sender<bool>) -> IoResult<()>
{
   tokio::signal::ctrl_c().await?;

   println!("\nReceived Ctrl+C, initiating shutdown...");

   // Send termination message to all listeners
   sender.send_replace(false);
   sender.closed().await;

   Ok(())
}

/// Program entry point.
#[tokio::main]
async fn main() -> IoResult<()>
{
   print_version();

   let (term_sender, mut term_receiver) = watch::channel(true);

   // Spawn a task to handle termination signal.
   let term_task =
      tokio::spawn(async move { listen_for_termination(term_sender).await });

   // Spawn a task to listen for network communication.
   let comm_task = tokio::spawn(async move {
      let mut running = true;
      while running
      {
         tokio::select! {
             _ = term_receiver.changed() => {
                 println!("Communication task received shutdown message.");
                 running = *term_receiver.borrow_and_update();
             }
         }
      }
   });

   // Wait for termination signal task and propagate any error.
   match term_task.await
   {
      Ok(Ok(())) =>
      {}
      Ok(Err(e)) => return Err(e),
      Err(e) =>
      {
         eprintln!("Termination task panicked: {}", e);
         return Err(io::Error::new(io::ErrorKind::Other,
                                   "Termination task panicked"));
      }
   }

   // Wait for the communication task to complete.
   let _ = comm_task.await;

   println!("Shutdown complete.");
   Ok(())
}
